package me.unurled.sacredrealms.sr.components.player;

import static me.unurled.sacredrealms.sr.utils.Logger.error;

import com.google.gson.Gson;
import java.util.HashMap;
import java.util.UUID;
import me.unurled.sacredrealms.sr.data.DataHandler;
import me.unurled.sacredrealms.sr.data.DataManager;
import me.unurled.sacredrealms.sr.managers.Manager;
import org.bukkit.event.EventHandler;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class PlayerManager extends Manager {

  private final HashMap<UUID, SRPlayer> players;

  public PlayerManager() {
    super();
    players = new HashMap<>();
  }

  /** Save the data */
  @Override
  public void saveData() {
    DataManager dm = (DataManager) DataManager.getInstance(DataManager.class);
    if (dm == null) {
      error("DataManager is null, Can't save player data.");
      return;
    }
    DataHandler dh = dm.getDataHandler();
    Gson gson = new Gson();
    dh.set("sr.players", gson.toJson(players));
  }

  /** Load the data */
  @Override
  public void loadData() {}

  @Nullable
  public SRPlayer getPlayer(UUID uuid) {
    return players.get(uuid);
  }

  public void addPlayer(@NotNull SRPlayer player) {
    players.put(player.getUuid(), player);
  }

  public void removePlayer(UUID uuid) {
    players.remove(uuid);
  }

  public boolean isSRPlayer(UUID uuid) {
    return players.containsKey(uuid);
  }

  @EventHandler
  public void onPlayerJoin(PlayerJoinEvent e) {
    SRPlayer player = new SRPlayer(e.getPlayer());
    // load data from db // TODO: test this after redis cache
    /*DataManager dm = (DataManager) DataManager.getInstance(DataManager.class);
    if (dm == null) {
      error("DataManager is null, Can't load player " + e.getPlayer().getName() + "'s data" + ".");
      return;
    }
    DataHandler dh = dm.getDataHandler();
    Gson gson = new Gson();
    Object json = dh.getGson("sr.players." + e.getPlayer().getUniqueId());*/
    addPlayer(player);
  }

  // TODO: test if it works, i have a doubt about the jsonAppend (key + path seems weird)
  @EventHandler
  public void onPlayerQuit(PlayerQuitEvent e) {
    SRPlayer player = getPlayer(e.getPlayer().getUniqueId());
    if (player == null) {
      return;
    }
    // save for player
    DataManager dm = (DataManager) DataManager.getInstance(DataManager.class);
    if (dm == null) {
      error("DataManager is null, Can't save player " + e.getPlayer().getName() + "'s data" + ".");
      return;
    }
    DataHandler dh = dm.getDataHandler();
    Gson gson = new Gson();
    dh.set("sr.players." + e.getPlayer().getUniqueId(), gson.toJson(player));

    removePlayer(e.getPlayer().getUniqueId());
  }
}
